//shading:         phong
//lighting model:  blinn
//lighting type:   directional

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tWV: WORLDVIEW;
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (EX9)

//light properties
float3 lDir <string uiname="Light Direction";> = {0, -5, 2};        //light direction in world space
float4 lAmb  : COLOR <String uiname="Ambient Color";>  = {0.15, 0.15, 0.15, 1};
float4 lDiff : COLOR <String uiname="Diffuse Color";>  = {0.85, 0.85, 0.85, 1};
float4 lSpec : COLOR <String uiname="Specular Color";> = {0.35, 0.35, 0.35, 1};
float lPower <String uiname="Power"; float uimin=0.0;> = 25.0;     //shininess of specular highlight

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;
float4x4 tColor <string uiname="Color Transform";>;

struct vs2ps
{
    float4 PosWVP: POSITION;
    float4 TexCd : TEXCOORD0;
    float3 LightDirV: TEXCOORD1;
    float3 NormV: TEXCOORD2;
    float3 ViewDirV: TEXCOORD3;
    float depth: TEXCOORD4;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

//vertex tweaks
float3 point1, point2;
float width = 0.25;

float amp <string uiname="Curve";
	float UIMin = 0.0;
	float UIMax = 2.0;> = 0.25;
	
float bend <string uiname="Bend";
	float UIMin = -0.5;
	float UIMax = 1.0;> = 0.0;

float gamma <string uiname="Gamma";
	float UIMin = -1.0;
	float UIMax = 1.0;> = 0.0;
	
float pitch;
//--
	
float3 rotate(float3 Pos, float pitch)   //thanks to tonfilm
{
const float PI=3.1415926535;

    float3 vec = normalize(float3(0,sin(pitch*2*PI),cos(pitch*2*PI)));
    if ((vec.z==0.0)) {vec.z=0.0001;}
    float angle=atan2(vec.y,vec.z);
    //rotationmatrix
    float3x3 m;
    m[0][0]=1.;
    m[0][1]=0.;
    m[0][2]=0.;

    m[1][0]=0.;
    m[1][1]=cos(angle);
    m[1][2]=-sin(angle);

    m[2][0]=0.;
    m[2][1]=sin(angle);
    m[2][2]=cos(angle);
    Pos=mul(Pos,m);
    return Pos;
}

struct PosNorm
{
float4 Pos;
float3 Norm;
};

PosNorm segment(float4 Pos) {
const float PI=3.1415926535;

       float max = 0.5*abs(amp)*PI;
       float range=Pos.x*abs(amp)*PI;
       float x_clamp=sin(max)/0.5;
       
       float zero_one=Pos.x+0.5;
       float sinus=(pow(zero_one,2)-zero_one)*4;



       Pos.x=((sin(range+(gamma*sinus))/x_clamp)-(gamma*sinus));//*distance(point1,point2);
       Pos.y*=width*(1.-((bend*sinus)+abs(bend)));
       Pos.z+=(((cos(range)-cos(max))/x_clamp))*sign(amp);//*distance(point1,point2);
       Pos.xyz=rotate(Pos.xyz,pitch);

       float3 max_const=float3(0.5,0,0);
       float3 diff=lerp(point1+max_const,point2-max_const,zero_one);

       Pos.xyz+=diff;

       float3 Norm=0;
       Norm.x=(sin(range+(gamma*sinus)))*sign(amp);
       Norm.z=(cos(range+(gamma*sinus)))*sign(amp);
       Norm=rotate(Norm,pitch);

       PosNorm Out;
       Out.Pos=Pos;
       Out.Norm=Norm;
       return Out;
}

vs2ps VS(
    float4 PosO: POSITION,
    float3 NormO: NORMAL,
    float4 TexCd : TEXCOORD0,
    float depth: TEXCOORD1)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));
    
    //normal in view space
    Out.NormV = normalize(mul(segment(PosO).Norm, tWV));

    //position (projected)
    Out.PosWVP  = mul(segment(PosO).Pos, tWVP);
    Out.TexCd = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    Out.depth=Out.PosWVP.z;
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------
//pixelshader tweaks
float depth_min, depth_max = 5.0, alpha = 1.0;
//--

float4 PS(vs2ps In): COLOR
{
    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(In.NormV, In.LightDirV), dot(In.NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(In.NormV, In.LightDirV) * In.NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    col.rgb *= 1.-clamp((In.depth-depth_min)/(depth_max-depth_min),0,1);
    col.a*=alpha;
    //col.a*=saturate(In.depth);

    return mul(col, tColor);
}


// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TPhongDirectional
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader = compile ps_2_0 PS();
    }
}

technique TFallbackGouraudDirectionalFF
{
    pass P0
    {
        //transformations
        NormalizeNormals = true;
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //material
        MaterialAmbient  = {1, 1, 1, 1};
        MaterialDiffuse  = {1, 1, 1, 1};
        MaterialSpecular = {1, 1, 1, 1};
        MaterialPower    = (lPower);

        //texturing
        Sampler[0] = (Samp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere

        //lighting
        LightEnable[0] = TRUE;
        Lighting       = TRUE;
        SpecularEnable = TRUE;

        LightType[0]     = DIRECTIONAL;
        LightAmbient[0]  = (lAmb);
        LightDiffuse[0]  = (lDiff);
        LightSpecular[0] = (lSpec);
        LightDirection[0] = (lDir);

        //shading
        ShadeMode = GOURAUD;
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
